@@ -23,13 +23,11 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
-
+from __future__ import absolute_import
 
 from traits.api import (Bool, Delegate, Float, Instance, List, Property, Range,
                         ReadOnly, Str, TraitError, cached_property)
 
-import traits._py2to3 as _py2to3
-
 from .view_element import ViewSubElement
 
 from .item import Item
@@ -38,7 +36,10 @@ from .include import Include
 
 from .ui_traits import SequenceTypes, ContainerDelegate, Orientation, Layout
 
+from .util import str_find
+
 from .dock_window_theme import dock_window_theme, DockWindowTheme
+import six
 
 #-------------------------------------------------------------------------
 #  Trait definitions:
@@ -202,7 +203,7 @@ class Group(ViewSubElement):
 
         # Process any embedded Group options first:
         for value in values:
-            if (isinstance(value, str)) and (value[0:1] in '-|'):
+            if (isinstance(value, six.string_types)) and (value[0:1] in '-|'):
                 # Parse Group trait options if specified as a string:
                 self._parse(value)
 
@@ -213,7 +214,7 @@ class Group(ViewSubElement):
             elif type(value) in SequenceTypes:
                 # Map (...) or [...] to a Group():
                 content.append(Group(*value))
-            elif isinstance(value, str):
+            elif isinstance(value, six.string_types):
                 if value[0:1] in '-|':
                     # We've already parsed Group trait options above:
                     pass
@@ -365,7 +366,7 @@ class Group(ViewSubElement):
         self.show_labels = show_labels
 
         # Parse all of the punctuation based sub-string options:
-        value = self._split('id', value, ':', _py2to3.str_find, 0, 1)
+        value = self._split('id', value, ':', str_find, 0, 1)
         if value != '':
             self.object = value
 
@@ -784,7 +785,7 @@ class ShadowGroup(Group):
         if len(items) > 0:
             content.append(
                 # Set shadow before hand to prevent delegation errors
-                ShadowGroup(shadow=self.shadow).set(
+                ShadowGroup(shadow=self.shadow).trait_set(
                     groups=0,
                     label='',
                     show_border=False,
