@@ -22,12 +22,18 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import, print_function
+
 import datetime
 
 from pyface.qt import QtCore, QtGui
+from traits.api import Bool, Date, Dict, Instance, Set
 
 from .editor import Editor
 from .editor_factory import ReadonlyEditor as BaseReadonlyEditor
+from traitsui.editors.date_editor import CellFormat
+
+import six
 
 #-------------------------------------------------------------------------
 #  'SimpleEditor' class:
@@ -98,14 +104,7 @@ class SimpleEditor(Editor):
     def update_object(self, q_date):
         """ Handles the user entering input data in the edit control.
         """
-        year = q_date.year()
-        month = q_date.month()
-        day = q_date.day()
-        try:
-            self.value = datetime.date(year, month, day)
-        except ValueError:
-            print('Invalid date:', year, month, day)
-            raise
+        self.value = datetime.date(q_date.year(), q_date.month(), q_date.day())
 
 #-------------------------------------------------------------------------
 #  'CustomEditor' class:
@@ -116,6 +115,13 @@ class CustomEditor(Editor):
     """ Custom Traits UI date editor that wraps QCalendarWidget.
     """
 
+    # Style used for when a date is unselected.
+    # Mapping from datetime.date to CellFormat
+    _unselected_styles = Dict(Date, Instance(CellFormat))
+
+    # Selected dates (used when multi_select is true)
+    _selected = Set(Date)
+
     #-------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
     #  widget:
@@ -142,8 +148,14 @@ class CustomEditor(Editor):
         """
         value = self.value
         if value:
-            q_date = QtCore.QDate(value.year, value.month, value.day)
-            self.control.setSelectedDate(q_date)
+            if not self.factory.multi_select:
+                q_date = QtCore.QDate(value.year, value.month, value.day)
+                self.control.setSelectedDate(q_date)
+            else:
+                self.apply_unselected_style_to_all()
+                for date in value:
+                    self.apply_style(self.factory.selected_style, date)
+                self._selected = set(value)
 
     #-------------------------------------------------------------------------
     #  Handles the user entering input data in the edit control:
@@ -152,14 +164,60 @@ class CustomEditor(Editor):
     def update_object(self, q_date):
         """ Handles the user entering input data in the edit control.
         """
-        year = q_date.year()
-        month = q_date.month()
-        day = q_date.day()
-        try:
-            self.value = datetime.date(year, month, day)
-        except ValueError:
-            print('Invalid date:', year, month, day)
-            raise
+        value = datetime.date(q_date.year(), q_date.month(), q_date.day())
+        if self.factory.multi_select:
+            if value in self.value:
+                self.unselect_date(value)
+            else:
+                self.select_date(value)
+
+            self.value = sorted(self._selected)
+
+        else:
+            self.value = value
+
+    def unselect_date(self, date):
+        self._selected.remove(date)
+        self.apply_unselected_style(date)
+
+    def select_date(self, date):
+        self._selected.add(date)
+        self.apply_style(self.factory.selected_style, date)
+
+    #-------------------------------------------------------------------------
+    #  Helper methods for applying styling
+    #-------------------------------------------------------------------------
+
+    def set_unselected_style(self, style, date):
+        """ Set the style used for a date when it is not selected."""
+        self._unselected_styles[date] = style
+        if self.factory.multi_select:
+            if date not in self.value:
+                self.apply_style(style, date)
+        else:
+            self.apply_style(style, date)
+
+    def apply_style(self, style, date):
+        """ Apply a given style to a given date."""
+        qdt = QtCore.QDate(date)
+        textformat = self.control.dateTextFormat(qdt)
+        _apply_cellformat(style, textformat)
+        self.control.setDateTextFormat(qdt, textformat)
+
+    def apply_unselected_style(self, date):
+        """ Apply the style for when a date is unselected."""
+        # Resets the text format on the given dates
+        textformat = QtGui.QTextCharFormat()
+        if date in self._unselected_styles:
+            _apply_cellformat(self._unselected_styles[date], textformat)
+        qdt = QtCore.QDate(date)
+        self.control.setDateTextFormat(qdt, textformat)
+
+    def apply_unselected_style_to_all(self):
+        """ Make all the selected dates appear unselected.
+        """
+        for date in self._selected:
+            self.apply_unselected_style(date)
 
 #------------------------------------------------------------------------------
 # 'ReadonlyEditor' class:
@@ -177,3 +235,71 @@ class ReadonlyEditor(BaseReadonlyEditor):
             return self.factory.message
         else:
             return self.value.strftime(self.factory.strftime)
+
+
+#------------------------------------------------------------------------
+# Helper functions for styling
+#------------------------------------------------------------------------
+
+
+def _apply_cellformat(cf, textformat):
+    """ Applies the formatting in the cellformat cf to the QTextCharFormat
+    object provided.
+    """
+    if cf.italics is not None:
+        textformat.setFontItalic(cf.italics)
+
+    if cf.underline is not None:
+        textformat.setFontUnderline(cf.underline)
+
+    if cf.bold is not None:
+        if cf.bold:
+            weight = QtGui.QFont.Bold
+        else:
+            weight = QtGui.QFont.Normal
+        textformat.setFontWeight(weight)
+
+    if cf.bgcolor is not None:
+        textformat.setBackground(_color_to_brush(cf.bgcolor))
+
+    if cf.fgcolor is not None:
+        textformat.setForeground(_color_to_brush(cf.fgcolor))
+
+
+def _textformat_to_cellformat(textformat):
+    """ Convert QTextCharFormat to a CellFormat """
+    bg_brush = textformat.background()
+    fg_brush = textformat.foreground()
+    return CellFormat(
+        italics=textformat.fontItalic(),
+        underline=textformat.fontUnderline(),
+        bold=textformat.fontWeight() == QtGui.QFont.Bold,
+        bgcolor=_brush_to_color(bg_brush),
+        fgcolor=_brush_to_color(fg_brush),
+    )
+
+
+def _color_to_brush(color):
+    """ Returns a QBrush with the color specified in **color** """
+    brush = QtGui.QBrush()
+    if isinstance(color, six.string_types) and hasattr(QtCore.Qt, color):
+        col = getattr(QtCore.Qt, color)
+    elif isinstance(color, tuple):
+        col = QtGui.QColor()
+        col.setRgb(*color[:4])
+    else:
+        raise RuntimeError("Invalid color specification '%r'" % color)
+
+    brush.setColor(col)
+    return brush
+
+
+def _brush_to_color(brush):
+    if brush.style() == 0:   # Qt.BrushStyle.NoBrush
+        return None
+
+    color = brush.color()
+    return (
+        color.red(), color.green(), color.blue(), color.alpha()
+    )
+
