@@ -13,49 +13,39 @@
 """ Defines the various editors and the editor factory for single-selection
     enumerations, for the PyQt user interface toolkit.
 """
+from __future__ import absolute_import
 
-#-------------------------------------------------------------------------
-#  Imports:
-#-------------------------------------------------------------------------
+from functools import reduce
+
+import six
+from six.moves import range
 
 from pyface.qt import QtCore, QtGui
 
-from traits.api \
-    import Bool, Property
-
+from traits.api import Bool, Property
 # FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
 # compatibility. The class has been moved to the
 # traitsui.editors.enum_editor file.
-from traitsui.editors.enum_editor \
-    import ToolkitEditorFactory
-
-from .editor \
-    import Editor
-
-from .constants \
-    import OKColor, ErrorColor
-
-from traitsui.helper \
-    import enum_values_changed
-from functools import reduce
+from traitsui.editors.enum_editor import ToolkitEditorFactory
+from traitsui.helper import enum_values_changed
+from .constants import OKColor, ErrorColor
+from .editor import Editor
 
 
 # default formatting function (would import from string, but not in Python 3)
 capitalize = lambda s: s.capitalize()
 
 
-#-------------------------------------------------------------------------
-#  'BaseEditor' class:
-#-------------------------------------------------------------------------
+completion_mode_map = {
+    'popup': QtGui.QCompleter.PopupCompletion,
+    'inline': QtGui.QCompleter.InlineCompletion,
+}
+
 
 class BaseEditor(Editor):
     """ Base class for enumeration editors.
     """
 
-    #-------------------------------------------------------------------------
-    #  Trait definitions:
-    #-------------------------------------------------------------------------
-
     # Current set of enumeration names:
     names = Property
 
@@ -66,8 +56,25 @@ class BaseEditor(Editor):
     inverse_mapping = Property
 
     #-------------------------------------------------------------------------
-    #  Finishes initializing the editor by creating the underlying toolkit
-    #  widget:
+    #  BaseEditor Interface
+    #-------------------------------------------------------------------------
+
+    def values_changed(self):
+        """ Recomputes the cached data based on the underlying enumeration model.
+        """
+        self._names, self._mapping, self._inverse_mapping = \
+            enum_values_changed(self._value(), self.string_value)
+
+    def rebuild_editor(self):
+        """ Rebuilds the contents of the editor whenever the original factory
+            object's **values** trait changes.
+
+            This is not needed for the Qt backends.
+        """
+        raise NotImplementedError
+
+    #-------------------------------------------------------------------------
+    #  Editor Interface
     #-------------------------------------------------------------------------
 
     def init(self, parent):
@@ -85,77 +92,6 @@ class BaseEditor(Editor):
             factory.on_trait_change(self.rebuild_editor, 'values_modified',
                                     dispatch='ui')
 
-    #-------------------------------------------------------------------------
-    #  Gets the current set of enumeration names:
-    #-------------------------------------------------------------------------
-
-    def _get_names(self):
-        """ Gets the current set of enumeration names.
-        """
-        if self._object is None:
-            return self.factory._names
-
-        return self._names
-
-    #-------------------------------------------------------------------------
-    #  Gets the current mapping:
-    #-------------------------------------------------------------------------
-
-    def _get_mapping(self):
-        """ Gets the current mapping.
-        """
-        if self._object is None:
-            return self.factory._mapping
-
-        return self._mapping
-
-    #-------------------------------------------------------------------------
-    #  Gets the current inverse mapping:
-    #-------------------------------------------------------------------------
-
-    def _get_inverse_mapping(self):
-        """ Gets the current inverse mapping.
-        """
-        if self._object is None:
-            return self.factory._inverse_mapping
-
-        return self._inverse_mapping
-
-    #-------------------------------------------------------------------------
-    #  Rebuilds the contents of the editor whenever the original factory
-    #  object's 'values' trait changes:
-    #-------------------------------------------------------------------------
-
-    def rebuild_editor(self):
-        """ Rebuilds the contents of the editor whenever the original factory
-            object's **values** trait changes.
-        """
-        raise NotImplementedError
-
-    #-------------------------------------------------------------------------
-    #  Recomputes the cached data based on the underlying enumeration model:
-    #-------------------------------------------------------------------------
-
-    def values_changed(self):
-        """ Recomputes the cached data based on the underlying enumeration model.
-        """
-        self._names, self._mapping, self._inverse_mapping = \
-            enum_values_changed(self._value(), self.string_value)
-
-    #-------------------------------------------------------------------------
-    #  Handles the underlying object model's enumeration set being changed:
-    #-------------------------------------------------------------------------
-
-    def _values_changed(self):
-        """ Handles the underlying object model's enumeration set being changed.
-        """
-        self.values_changed()
-        self.rebuild_editor()
-
-    #-------------------------------------------------------------------------
-    #  Disposes of the contents of an editor:
-    #-------------------------------------------------------------------------
-
     def dispose(self):
         """ Disposes of the contents of an editor.
         """
@@ -168,9 +104,43 @@ class BaseEditor(Editor):
 
         super(BaseEditor, self).dispose()
 
-#-------------------------------------------------------------------------
-#  'SimpleEditor' class:
-#-------------------------------------------------------------------------
+    #-------------------------------------------------------------------------
+    #  Private interface
+    #-------------------------------------------------------------------------
+
+    # Trait default handlers -------------------------------------------------
+
+    def _get_names(self):
+        """ Gets the current set of enumeration names.
+        """
+        if self._object is None:
+            return self.factory._names
+
+        return self._names
+
+    def _get_mapping(self):
+        """ Gets the current mapping.
+        """
+        if self._object is None:
+            return self.factory._mapping
+
+        return self._mapping
+
+    def _get_inverse_mapping(self):
+        """ Gets the current inverse mapping.
+        """
+        if self._object is None:
+            return self.factory._inverse_mapping
+
+        return self._inverse_mapping
+
+    # Trait change handlers --------------------------------------------------
+
+    def _values_changed(self):
+        """ Handles the underlying object model's enumeration set being changed.
+        """
+        self.values_changed()
+        self.rebuild_editor()
 
 
 class SimpleEditor(BaseEditor):
@@ -178,8 +148,7 @@ class SimpleEditor(BaseEditor):
     """
 
     #-------------------------------------------------------------------------
-    #  Finishes initializing the editor by creating the underlying toolkit
-    #  widget:
+    #  Editor Interface
     #-------------------------------------------------------------------------
 
     def init(self, parent):
@@ -195,6 +164,8 @@ class SimpleEditor(BaseEditor):
 
         if self.factory.evaluate is not None:
             control.setEditable(True)
+            control.completer().setCompletionMode(
+                completion_mode_map[self.factory.completion_mode])
             if self.factory.auto_set:
                 control.editTextChanged.connect(self.update_text_object)
             else:
@@ -205,8 +176,54 @@ class SimpleEditor(BaseEditor):
         self._no_enum_update = 0
         self.set_tooltip()
 
+    def update_editor(self):
+        """ Updates the editor when the object trait changes externally to the
+            editor.
+        """
+        if self._no_enum_update == 0:
+            self._no_enum_update += 1
+            if self.factory.evaluate is None:
+                try:
+                    index = self.names.index(self.inverse_mapping[self.value])
+                    self.control.setCurrentIndex(index)
+                except:
+                    self.control.setCurrentIndex(-1)
+            else:
+                try:
+                    self.control.setEditText(self.str_value)
+                except:
+                    self.control.setEditText('')
+            self._no_enum_update -= 1
+
+    def error(self, excp):
+        """ Handles an error that occurs while setting the object's trait value.
+        """
+        self._set_background(ErrorColor)
+
+    def rebuild_editor(self):
+        """ Rebuilds the contents of the editor whenever the original factory
+            object's **values** trait changes.
+        """
+        self.control.blockSignals(True)
+        try:
+            self.control.clear()
+            self.control.addItems(self.names)
+        finally:
+            self.control.blockSignals(False)
+
+        self.update_editor()
+
+    def set_size_policy(self, direction, resizable, springy, stretch):
+        super(SimpleEditor, self).set_size_policy(
+            direction, resizable, springy, stretch)
+
+        if ((direction == QtGui.QBoxLayout.LeftToRight and springy) or
+                (direction != QtGui.QBoxLayout.LeftToRight and resizable)):
+            self.control.setSizeAdjustPolicy(
+                QtGui.QComboBox.AdjustToContentsOnFirstShow)
+
     #-------------------------------------------------------------------------
-    #  Returns the QComboBox used for the editor control:
+    #  Private interface
     #-------------------------------------------------------------------------
 
     def create_combo_box(self):
@@ -218,27 +235,13 @@ class SimpleEditor(BaseEditor):
                               QtGui.QSizePolicy.Fixed)
         return control
 
-    #-------------------------------------------------------------------------
-    #  Adjust size policy to behave properly in group
-    #-------------------------------------------------------------------------
+    def _set_background(self, col):
+        le = self.control.lineEdit()
+        pal = QtGui.QPalette(le.palette())
+        pal.setColor(QtGui.QPalette.Base, col)
+        le.setPalette(pal)
 
-    def set_size_policy(self, direction, resizable, springy, stretch):
-        super(
-            SimpleEditor,
-            self).set_size_policy(
-            direction,
-            resizable,
-            springy,
-            stretch)
-
-        if ((direction == QtGui.QBoxLayout.LeftToRight and springy) or
-                (direction != QtGui.QBoxLayout.LeftToRight and resizable)):
-            self.control.setSizeAdjustPolicy(
-                QtGui.QComboBox.AdjustToContentsOnFirstShow)
-
-    #-------------------------------------------------------------------------
-    #  Handles the user selecting a new value from the combo box:
-    #-------------------------------------------------------------------------
+    #  Signal handlers -------------------------------------------------------
 
     def update_object(self, text):
         """ Handles the user selecting a new value from the combo box.
@@ -246,24 +249,21 @@ class SimpleEditor(BaseEditor):
         if self._no_enum_update == 0:
             self._no_enum_update += 1
             try:
-                self.value = self.mapping[str(text)]
-            except:
+                self.value = self.mapping[six.text_type(text)]
+            except Exception:
                 from traitsui.api import raise_to_debug
                 raise_to_debug()
             self._no_enum_update -= 1
 
-    #-------------------------------------------------------------------------
-    #  Handles the user typing text into the combo box text entry field:
-    #-------------------------------------------------------------------------
-
     def update_text_object(self, text):
         """ Handles the user typing text into the combo box text entry field.
         """
         if self._no_enum_update == 0:
-            value = str(text)
+
+            value = six.text_type(text)
             try:
                 value = self.mapping[value]
-            except:
+            except Exception:
                 try:
                     value = self.factory.evaluate(value)
                 except Exception as excp:
@@ -286,68 +286,6 @@ class SimpleEditor(BaseEditor):
             text = self.control.lineEdit().text()
             return self.update_text_object(text)
 
-    #-------------------------------------------------------------------------
-    #  Updates the editor when the object trait changes external to the editor:
-    #-------------------------------------------------------------------------
-
-    def update_editor(self):
-        """ Updates the editor when the object trait changes externally to the
-            editor.
-        """
-        if self._no_enum_update == 0:
-            self._no_enum_update += 1
-            if self.factory.evaluate is None:
-                try:
-                    index = self.names.index(self.inverse_mapping[self.value])
-                    self.control.setCurrentIndex(index)
-                except:
-                    self.control.setCurrentIndex(-1)
-            else:
-                try:
-                    self.control.setEditText(self.str_value)
-                except:
-                    self.control.setEditText('')
-            self._no_enum_update -= 1
-
-    #-------------------------------------------------------------------------
-    #  Handles an error that occurs while setting the object's trait value:
-    #-------------------------------------------------------------------------
-
-    def error(self, excp):
-        """ Handles an error that occurs while setting the object's trait value.
-        """
-        self._set_background(ErrorColor)
-
-    #-------------------------------------------------------------------------
-    #  Sets the background color of the QLineEdit of the QComboBox.
-    #-------------------------------------------------------------------------
-
-    def _set_background(self, col):
-        le = self.control.lineEdit()
-        pal = QtGui.QPalette(le.palette())
-        pal.setColor(QtGui.QPalette.Base, col)
-        le.setPalette(pal)
-
-    #-------------------------------------------------------------------------
-    #  Rebuilds the contents of the editor whenever the original factory
-    #  object's 'values' trait changes:
-    #-------------------------------------------------------------------------
-
-    def rebuild_editor(self):
-        """ Rebuilds the contents of the editor whenever the original factory
-            object's **values** trait changes.
-        """
-        self.control.blockSignals(True)
-        self.control.clear()
-        self.control.addItems(self.names)
-        self.control.blockSignals(False)
-
-        self.update_editor()
-
-#-------------------------------------------------------------------------
-#  'RadioEditor' class:
-#-------------------------------------------------------------------------
-
 
 class RadioEditor(BaseEditor):
     """ Enumeration editor, used for the "custom" style, that displays radio
@@ -358,8 +296,7 @@ class RadioEditor(BaseEditor):
     row_major = Bool(False)
 
     #-------------------------------------------------------------------------
-    #  Finishes initializing the editor by creating the underlying toolkit
-    #  widget:
+    #  Editor Interface
     #-------------------------------------------------------------------------
 
     def init(self, parent):
@@ -377,22 +314,6 @@ class RadioEditor(BaseEditor):
 
         self.rebuild_editor()
 
-    #-------------------------------------------------------------------------
-    #  Handles the user clicking one of the 'custom' radio buttons:
-    #-------------------------------------------------------------------------
-
-    def update_object(self, index):
-        """ Handles the user clicking one of the custom radio buttons.
-        """
-        try:
-            self.value = self.mapping[self.names[index]]
-        except:
-            pass
-
-    #-------------------------------------------------------------------------
-    #  Updates the editor when the object trait changes external to the editor:
-    #-------------------------------------------------------------------------
-
     def update_editor(self):
         """ Updates the editor when the object trait changes externally to the
             editor.
@@ -403,11 +324,6 @@ class RadioEditor(BaseEditor):
             rb = layout.itemAt(i).widget()
             rb.setChecked(rb.value == value)
 
-    #-------------------------------------------------------------------------
-    #  Rebuilds the contents of the editor whenever the original factory
-    #  object's 'values' trait changes:
-    #-------------------------------------------------------------------------
-
     def rebuild_editor(self):
         """ Rebuilds the contents of the editor whenever the original factory
             object's **values** trait changes.
@@ -455,7 +371,7 @@ class RadioEditor(BaseEditor):
                     n -= 1
 
     #-------------------------------------------------------------------------
-    #  Returns the QAbstractButton used for the radio button:
+    #  Private interface
     #-------------------------------------------------------------------------
 
     def create_button(self, name):
@@ -464,18 +380,24 @@ class RadioEditor(BaseEditor):
         label = self.string_value(name, capitalize)
         return QtGui.QRadioButton(label)
 
-#-------------------------------------------------------------------------
-#  'ListEditor' class:
-#-------------------------------------------------------------------------
+    #  Signal handlers -------------------------------------------------------
+
+    def update_object(self, index):
+        """ Handles the user clicking one of the custom radio buttons.
+        """
+        try:
+            self.value = self.mapping[self.names[index]]
+        except:
+            pass
 
 
 class ListEditor(BaseEditor):
     """ Enumeration editor, used for the "custom" style, that displays a list
         box.
     """
+
     #-------------------------------------------------------------------------
-    #  Finishes initializing the editor by creating the underlying toolkit
-    #  widget:
+    #  Editor Interface
     #-------------------------------------------------------------------------
 
     def init(self, parent):
@@ -490,30 +412,6 @@ class ListEditor(BaseEditor):
         self.rebuild_editor()
         self.set_tooltip()
 
-    #-------------------------------------------------------------------------
-    #  Handles the user selecting a list box item:
-    #-------------------------------------------------------------------------
-
-    def update_object(self, text):
-        """ Handles the user selecting a list box item.
-        """
-        value = str(text)
-        try:
-            value = self.mapping[value]
-        except:
-            try:
-                value = self.factory.evaluate(value)
-            except:
-                pass
-        try:
-            self.value = value
-        except:
-            pass
-
-    #-------------------------------------------------------------------------
-    #  Updates the editor when the object trait changes external to the editor:
-    #-------------------------------------------------------------------------
-
     def update_editor(self):
         """ Updates the editor when the object trait changes externally to the
             editor.
@@ -532,11 +430,6 @@ class ListEditor(BaseEditor):
         except:
             pass
 
-    #-------------------------------------------------------------------------
-    #  Rebuilds the contents of the editor whenever the original factory
-    #  object's 'values' trait changes:
-    #-------------------------------------------------------------------------
-
     def rebuild_editor(self):
         """ Rebuilds the contents of the editor whenever the original factory
             object's **values** trait changes.
@@ -549,3 +442,25 @@ class ListEditor(BaseEditor):
         self.control.blockSignals(False)
 
         self.update_editor()
+
+    #-------------------------------------------------------------------------
+    #  Private interface
+    #-------------------------------------------------------------------------
+
+    #  Signal handlers -------------------------------------------------------
+
+    def update_object(self, text):
+        """ Handles the user selecting a list box item.
+        """
+        value = unicode(text)
+        try:
+            value = self.mapping[value]
+        except:
+            try:
+                value = self.factory.evaluate(value)
+            except:
+                pass
+        try:
+            self.value = value
+        except:
+            pass
