@@ -18,6 +18,7 @@ PyQt user interface toolkit.
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
 from math \
     import log10
 
@@ -43,6 +44,7 @@ from .constants \
 
 from .helper \
     import IconButton
+import six
 
 #-------------------------------------------------------------------------
 #  'BaseRangeEditor' class:
@@ -207,12 +209,12 @@ class SimpleSliderEditor(BaseRangeEditor):
         """
         try:
             try:
-                value = eval(str(self.control.text.text()).strip())
+                value = eval(six.text_type(self.control.text.text()).strip())
             except Exception as ex:
                 # The entered something that didn't eval as a number, (e.g.,
                 # 'foo') pretend it didn't happen
                 value = self.value
-                self.control.text.setText(str(value))
+                self.control.text.setText(six.text_type(value))
                 # for compound editor, value may be non-numeric
                 if not isinstance(value, (int, float)):
                     return
@@ -495,7 +497,7 @@ class LargeRangeSliderEditor(BaseRangeEditor):
         """ Handles the user pressing the Enter key in the text field.
         """
         try:
-            self.value = eval(str(self.control.text.text()).strip())
+            self.value = eval(six.text_type(self.control.text.text()).strip())
         except TraitError as excp:
             pass
 
@@ -790,17 +792,18 @@ class RangeTextEditor(TextEditor):
         """ Handles the user entering input data in the edit control.
         """
         try:
-            value = eval(str(self.control.text()))
+            value = eval(six.text_type(self.control.text()))
             if self.evaluate is not None:
                 value = self.evaluate(value)
             self.value = value
             col = OKColor
-        except:
+        except Exception:
             col = ErrorColor
 
-        pal = QtGui.QPalette(self.control.palette())
-        pal.setColor(QtGui.QPalette.Base, col)
-        self.control.setPalette(pal)
+        if self.control is not None:
+            pal = QtGui.QPalette(self.control.palette())
+            pal.setColor(QtGui.QPalette.Base, col)
+            self.control.setPalette(pal)
 
 #-------------------------------------------------------------------------
 #  'SimpleEnumEditor' factory adaptor:
