@@ -13,10 +13,12 @@
 #-------------------------------------------------------------------------
 
 # System library imports
+from __future__ import absolute_import
 from pyface.qt import QtCore, QtGui
 
 # ETS imports
 from .editor import Editor
+import six
 
 
 class SearchWidget(QtGui.QLineEdit):
@@ -28,7 +30,7 @@ class SearchWidget(QtGui.QLineEdit):
         """ Store the descriptive text for the widget.
         """
         super(SearchWidget, self).__init__()
-        self._desc = str(desc)
+        self._desc = six.text_type(desc)
         self._set_descriptive_text()
 
     def focusInEvent(self, event):
