@@ -2,8 +2,10 @@
 # Copyright (c) 2008, Riverbank Computing Limited
 # All rights reserved.
 #
-# This software is provided without warranty under the terms of the BSD license.
-# However, when used with the GPL version of PyQt the additional terms described
+# This software is provided without warranty under the terms of the BSD
+# license.
+#
+# When used with the GPL version of PyQt the additional terms described
 # in the PyQt GPL exception also apply.
 #
 # Author: Riverbank Computing Limited
@@ -16,7 +18,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
-
+from __future__ import absolute_import
 
 from pyface.qt import QtCore, QtGui
 from pyface.image_resource import ImageResource
@@ -34,6 +36,7 @@ from traitsui.ui_traits import SequenceTypes
 
 from .editor import Editor
 from .table_model import TableModel, SortFilterTableModel
+import six
 
 
 is_qt5 = QtCore.__version_info__ >= (5,)
@@ -145,11 +148,14 @@ class TableEditor(Editor, BaseTableEditor):
         widget."""
 
         factory = self.factory
+        self.filter = factory.filter
+
         columns = factory.columns[:]
         if (len(columns) == 0) and (len(self.value) > 0):
             columns = [ObjectColumn(name=name)
                        for name in self.value[0].editable_traits()]
         self.columns = columns
+
         if factory.table_view_factory is not None:
             self.table_view = factory.table_view_factory(editor=self)
         if factory.source_model_factory is not None:
@@ -223,7 +229,7 @@ class TableEditor(Editor, BaseTableEditor):
 
         # Create auxillary editor and encompassing splitter if necessary
         mode = factory.selection_mode
-        if (factory.edit_view == ' ') or not mode in ('row', 'rows'):
+        if (factory.edit_view == ' ') or mode not in {'row', 'rows'}:
             self.control = main_view
         else:
             if factory.orientation == 'horizontal':
@@ -429,7 +435,7 @@ class TableEditor(Editor, BaseTableEditor):
         old = self._no_notify
         self._no_notify = True
         try:
-            for name, value in list(keywords.items()):
+            for name, value in keywords.items():
                 setattr(self, name, value)
         finally:
             self._no_notify = old
@@ -441,7 +447,7 @@ class TableEditor(Editor, BaseTableEditor):
     def set_selection(self, objects=[], notify=True):
         """Sets the current selection to a set of specified objects."""
 
-        if not isinstance(objects, SequenceTypes):
+        if not isinstance(objects, list):
             objects = [objects]
 
         mode = self.factory.selection_mode
@@ -456,7 +462,7 @@ class TableEditor(Editor, BaseTableEditor):
         # Selection mode is 'row' or 'rows'
         if mode.startswith('row'):
             flags |= QtGui.QItemSelectionModel.Rows
-            items = list(self.items())
+            items = self.items()
             for obj in objects:
                 try:
                     row = items.index(obj)
@@ -474,7 +480,7 @@ class TableEditor(Editor, BaseTableEditor):
 
         # Selection mode is 'cell' or 'cells'
         else:
-            items = list(self.items())
+            items = self.items()
             for obj, name in objects:
                 try:
                     row = items.index(obj)
@@ -486,15 +492,21 @@ class TableEditor(Editor, BaseTableEditor):
 
         # Perform the selection so that only one signal is emitted
         selection = QtGui.QItemSelection()
+        smodel = self.table_view.selectionModel()
+        if smodel is None:
+            # guard against selection during tear-down
+            return
         for index in indexes:
             index = self.model.mapFromSource(index)
             if index.isValid():
-                self.table_view.setCurrentIndex(index)
+                smodel.setCurrentIndex(
+                    index, QtGui.QItemSelectionModel.NoUpdate)
                 selection.select(index, index)
-        smodel = self.table_view.selectionModel()
+
+        smodel.blockSignals(not notify)
         try:
-            smodel.blockSignals(not notify)
             if len(selection.indexes()):
+                smodel.clear()
                 smodel.select(selection, flags)
             else:
                 smodel.clear()
@@ -528,7 +540,7 @@ class TableEditor(Editor, BaseTableEditor):
     def _update_filtering(self):
         """Update the filter summary and the filtered indices."""
 
-        items = list(self.items())
+        items = self.items()
         num_items = len(items)
 
         f = self.filter
@@ -556,7 +568,7 @@ class TableEditor(Editor, BaseTableEditor):
     def _get_image(self, image):
         """ Converts a user specified image to a QIcon.
         """
-        if isinstance(image, str):
+        if isinstance(image, six.string_types):
             self.image = image
             image = self.image
 
@@ -597,16 +609,36 @@ class TableEditor(Editor, BaseTableEditor):
         """Gets the row,column indices which match the selected trait"""
         selection_items = self.table_view.selectionModel().selection()
         indices = self.model.mapSelectionToSource(selection_items).indexes()
-        return [(index.row(), index.column()) for index in indices]
+        if self.factory.selection_mode.startswith('row'):
+            indices = sorted(set(index.row() for index in indices))
+        elif self.factory.selection_mode.startswith('column'):
+            indices = sorted(set(index.column() for index in indices))
+        else:
+            indices = [(index.row(), index.column()) for index in indices]
+
+        if self.factory.selection_mode in {'rows', 'columns', 'cells'}:
+            return indices
+        elif len(indices) > 0:
+            return indices[0]
+        else:
+            return -1
 
     def _set_selected_indices(self, indices):
+        if not isinstance(indices, list):
+            indices = [indices]
         selected = []
-        for row, col in indices:
-            selected.append((self.value[row], self.columns[col].name))
+        if self.factory.selection_mode.startswith('row'):
+            for row in indices:
+                selected.append(self.value[row])
+        elif self.factory.selection_mode.startswith('column'):
+            for col in indices:
+                selected.append(self.columns[col].name)
+        else:
+            for row, col in indices:
+                selected.append((self.value[row], self.columns[col].name))
 
         self.selected = selected
         self.set_selection(self.selected, False)
-        return
 
     #-- Trait Change Handlers ------------------------------------------------
 
@@ -629,8 +661,7 @@ class TableEditor(Editor, BaseTableEditor):
             if column.renderer:
                 self.table_view.setItemDelegateForColumn(i, column.renderer)
 
-        self.model.beginResetModel()
-        self.model.endResetModel()
+        self.model.invalidate()
         self.table_view.resizeColumnsToContents()
         if self.auto_size:
             self.table_view.resizeRowsToContents()
@@ -650,7 +681,7 @@ class TableEditor(Editor, BaseTableEditor):
     def _on_row_selection(self, added, removed):
         """Handle the row selection being changed."""
 
-        items = list(self.items())
+        items = self.items()
         indexes = self.table_view.selectionModel().selectedRows()
         if len(indexes):
             index = self.model.mapToSource(indexes[0])
@@ -663,8 +694,7 @@ class TableEditor(Editor, BaseTableEditor):
 
     def _on_rows_selection(self, added, removed):
         """Handle the rows selection being changed."""
-
-        items = list(self.items())
+        items = self.items()
         indexes = self.table_view.selectionModel().selectedRows()
         selected = [items[self.model.mapToSource(index).row()]
                     for index in indexes]
@@ -698,7 +728,7 @@ class TableEditor(Editor, BaseTableEditor):
     def _on_cell_selection(self, added, removed):
         """Handle the cell selection being changed."""
 
-        items = list(self.items())
+        items = self.items()
         indexes = self.table_view.selectionModel().selectedIndexes()
         if len(indexes):
             index = self.model.mapToSource(indexes[0])
@@ -715,7 +745,7 @@ class TableEditor(Editor, BaseTableEditor):
     def _on_cells_selection(self, added, removed):
         """Handle the cells selection being changed."""
 
-        items = list(self.items())
+        items = self.items()
         indexes = self.table_view.selectionModel().selectedIndexes()
         selected = []
         for index in indexes:
@@ -732,7 +762,7 @@ class TableEditor(Editor, BaseTableEditor):
 
         index = self.model.mapToSource(index)
         column = self.columns[index.column()]
-        obj = list(self.items())[index.row()]
+        obj = self.items()[index.row()]
 
         # Fire the same event on the editor after mapping it to a model object
         # and column name:
@@ -746,7 +776,7 @@ class TableEditor(Editor, BaseTableEditor):
 
         index = self.model.mapToSource(index)
         column = self.columns[index.column()]
-        obj = list(self.items())[index.row()]
+        obj = self.items()[index.row()]
 
         # Fire the same event on the editor after mapping it to a model object
         # and column name:
@@ -781,9 +811,11 @@ class TableEditor(Editor, BaseTableEditor):
 
         self.model.moveRow(self.header_row, self.header_row + 1)
 
+
 # Define the SimpleEditor class.
 SimpleEditor = TableEditor
 
+
 # Define the ReadonlyEditor class.
 ReadonlyEditor = TableEditor
 
@@ -803,7 +835,7 @@ class TableDelegate(QtGui.QStyledItemDelegate):
         index = model.mapToSource(index)
         table_editor = model._editor
         column = table_editor.columns[index.column()]
-        obj = list(table_editor.items())[index.row()]
+        obj = table_editor.items()[index.row()]
 
         factory = column.get_editor(obj)
         style = column.get_style(obj)
@@ -931,7 +963,7 @@ class TableView(QtGui.QTableView):
 
         # ...or show a context menu for a cell.
         elif column != -1:
-            obj = list(editor.items())[row]
+            obj = editor.items()[row]
             column = editor.columns[column]
             menu_manager = column.get_menu(obj)
             if menu_manager is None:
@@ -974,9 +1006,9 @@ class TableView(QtGui.QTableView):
 
     def resizeEvent(self, event):
         """Reimplemented to size the table columns when the size of the table
-        changes. Because the layout algorithm requires that the available space
-        be known, we have to wait until the UI that contains this table gives it
-        its initial size."""
+        changes. Because the layout algorithm requires that the available
+        space be known, we have to wait until the UI that contains this table
+        gives it its initial size."""
 
         QtGui.QTableView.resizeEvent(self, event)
 
@@ -987,7 +1019,8 @@ class TableView(QtGui.QTableView):
         else:
             parent = self.parent()
             if (not self._initial_size and parent and (
-                    self.isVisible() or isinstance(parent, QtGui.QMainWindow))):
+                    self.isVisible()
+                    or isinstance(parent, QtGui.QMainWindow))):
                 self._initial_size = True
                 if self._editor.auto_size:
                     self.resizeColumnsToContents()
@@ -1055,7 +1088,7 @@ class TableView(QtGui.QTableView):
             return requested_width
 
     def resizeColumnsToContents(self):
-        """Reimplemented to support proportional column width specifications."""
+        """ Support proportional column width specifications. """
 
         # TODO: The proportional size specification approach found in the
         # TableColumns is not entirely compatible with the ability to
@@ -1126,10 +1159,12 @@ class TableView(QtGui.QTableView):
         # we make the last non-fixed-size column stretchy.
         hheader = self.horizontalHeader()
         set_resize_mode = set_qheader_section_resize_mode(hheader)
-        resize_mode_map = dict(interactive=QtGui.QHeaderView.Interactive,
-                               fixed=QtGui.QHeaderView.Fixed,
-                               stretch=QtGui.QHeaderView.Stretch,
-                               resize_to_contents=QtGui.QHeaderView.ResizeToContents)
+        resize_mode_map = dict(
+            interactive=QtGui.QHeaderView.Interactive,
+            fixed=QtGui.QHeaderView.Fixed,
+            stretch=QtGui.QHeaderView.Stretch,
+            resize_to_contents=QtGui.QHeaderView.ResizeToContents,
+        )
         stretchable_columns = []
         for i, column in enumerate(editor.columns):
             set_resize_mode(i, resize_mode_map[column.resize_mode])
@@ -1184,34 +1219,56 @@ class TableFilterEditor(HasTraits):
     remove_button = Button('Delete')
 
     # The default view for this editor
-    view = View(Group(Group(Group(Item('add_button',
-                                       enabled_when='selected_template'),
-                                  Item('remove_button',
-                                       enabled_when='len(templates) > 1 and '
-                                       'selected_filter is not None'),
-                                  orientation='horizontal',
-                                  show_labels=False),
-                            Label('Base filter for new filters:'),
-                            Item('selected_template',
-                                 editor=EnumEditor(name='templates')),
-                            Item('selected_filter',
-                                 style='custom',
-                                 editor=EnumEditor(name='filters',
-                                                   mode='list')),
-                            show_labels=False),
-                      Item('selected_filter',
-                           width=0.75,
-                           style='custom',
-                           editor=InstanceEditor(view_name='selected_filter_view')),
-                      id='TableFilterEditorSplit',
-                      show_labels=False,
-                      layout='split',
-                      orientation='horizontal'),
-                id='traitsui.qt4.table_editor.TableFilterEditor',
-                buttons=['OK', 'Cancel'],
-                kind='livemodal',
-                resizable=True, width=800, height=400,
-                title='Customize filters')
+    view = View(
+        Group(
+            Group(
+                Group(
+                    Item(
+                        'add_button',
+                        enabled_when='selected_template'
+                    ),
+                    Item(
+                        'remove_button',
+                        enabled_when='len(templates) > 1 and '
+                                     'selected_filter is not None',
+                    ),
+                    orientation='horizontal',
+                    show_labels=False,
+                ),
+                Label('Base filter for new filters:'),
+                Item(
+                    'selected_template',
+                    editor=EnumEditor(name='templates')
+                ),
+                Item(
+                    'selected_filter',
+                    style='custom',
+                    editor=EnumEditor(
+                        name='filters',
+                        mode='list',
+                    )
+                ),
+                show_labels=False,
+            ),
+            Item(
+                'selected_filter',
+                width=0.75,
+                style='custom',
+                editor=InstanceEditor(view_name='selected_filter_view')
+            ),
+            id='TableFilterEditorSplit',
+            show_labels=False,
+            layout='split',
+            orientation='horizontal',
+        ),
+        id='traitsui.qt4.table_editor.TableFilterEditor',
+        buttons=['OK', 'Cancel'],
+        kind='livemodal',
+        resizable=True,
+        width=800,
+        height=400,
+        title='Customize filters',
+    )
 
     #-------------------------------------------------------------------------
     #  Private methods:
@@ -1226,7 +1283,7 @@ class TableFilterEditor(HasTraits):
             model = self.editor.model
             index = model.mapToSource(model.index(0, 0))
             if index.isValid():
-                obj = list(self.editor.items())[index.row()]
+                obj = self.editor.items()[index.row()]
             else:
                 obj = None
             view = self.selected_filter.edit_view(obj)
