@@ -16,12 +16,15 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
 from pyface.qt import QtCore, QtGui
 
 from traitsui.ui_traits import SequenceTypes
 
 from .clipboard import PyMimeData
 
+import six
+
 #-------------------------------------------------------------------------
 #  Constants:
 #-------------------------------------------------------------------------
@@ -74,7 +77,7 @@ class TableModel(QtCore.QAbstractTableModel):
     def rowCount(self, mi):
         """Reimplemented to return the number of rows."""
 
-        return len(list(self._editor.items()))
+        return len(self._editor.items())
 
     def columnCount(self, mi):
         """Reimplemented to return the number of columns."""
@@ -84,7 +87,7 @@ class TableModel(QtCore.QAbstractTableModel):
     def data(self, mi, role):
         """Reimplemented to return the data."""
 
-        obj = list(self._editor.items())[mi.row()]
+        obj = self._editor.items()[mi.row()]
         column = self._editor.columns[mi.column()]
 
         if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole:
@@ -156,7 +159,7 @@ class TableModel(QtCore.QAbstractTableModel):
         flags = QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled | \
             QtCore.Qt.ItemIsDragEnabled
 
-        obj = list(editor.items())[mi.row()]
+        obj = editor.items()[mi.row()]
         column = editor.columns[mi.column()]
 
         if editor.factory:
@@ -198,7 +201,7 @@ class TableModel(QtCore.QAbstractTableModel):
             obj = editor.create_new_row()
 
         self.beginInsertRows(parent, row, row)
-        editor.callx(list(editor.items()).insert, row, obj)
+        editor.callx(editor.items().insert, row, obj)
         self.endInsertRows()
         return True
 
@@ -206,7 +209,7 @@ class TableModel(QtCore.QAbstractTableModel):
         """Reimplemented to allow creation of new rows."""
 
         editor = self._editor
-        items = list(editor.items())
+        items = editor.items()
         self.beginInsertRows(parent, row, row + count - 1)
         for i in range(count):
             editor.callx(items.insert, row + i, editor.create_new_row())
@@ -218,7 +221,7 @@ class TableModel(QtCore.QAbstractTableModel):
         and drop."""
 
         editor = self._editor
-        items = list(editor.items())
+        items = editor.items()
         self.beginRemoveRows(parent, row, row + count - 1)
         for i in range(count):
             editor.callx(items.pop, row + i)
@@ -252,10 +255,10 @@ class TableModel(QtCore.QAbstractTableModel):
 
         # handle re-ordering via internal drags
         if editor.factory.reorderable:
-            rows = sorted(set([index.row() for index in indexes]))
-            data = QtCore.QByteArray(str(id(self)))
+            rows = sorted({index.row() for index in indexes})
+            data = QtCore.QByteArray(six.text_type(id(self)).encode('utf8'))
             for row in rows:
-                data.append(' %i' % row)
+                data.append((' %i' % row).encode('utf8'))
             mime_data.setData(mime_type, data)
         return mime_data
 
@@ -268,13 +271,13 @@ class TableModel(QtCore.QAbstractTableModel):
         # this is a drag from a table model?
         data = mime_data.data(mime_type)
         if not data.isNull() and action == QtCore.Qt.MoveAction:
-            id_and_rows = list(map(int, str(data).split(' ')))
+            id_and_rows = [int(s) for s in data.data().decode('utf8').split(' ')]
             table_id = id_and_rows[0]
             # is it from ourself?
             if table_id == id(self):
                 current_rows = id_and_rows[1:]
                 if not parent.isValid():
-                    row = len(list(self._editor.items())) - 1
+                    row = len(self._editor.items()) - 1
                 else:
                     row = parent.row()
 
@@ -290,7 +293,7 @@ class TableModel(QtCore.QAbstractTableModel):
                 column = parent.column()
 
             if row != -1 and column != - 1:
-                object = list(editor.items())[row]
+                object = editor.items()[row]
                 column = editor.columns[column]
                 if column.is_droppable(object, data):
                     column.set_value(object, data)
@@ -317,14 +320,14 @@ class TableModel(QtCore.QAbstractTableModel):
         """ Return the model data for the column as a list """
         editor = self._editor
         column_obj = editor.columns[column]
-        return [column_obj.get_value(item) for item in list(editor.items())]
+        return [column_obj.get_value(item) for item in editor.items()]
 
     def _get_rows_drag_value(self, rows):
         """ Returns the value to use when the specified rows are dragged or
             copied and pasted. The parameter *rows* is a list of row indexes.
             Return a list of objects.
         """
-        items = list(self._editor.items())
+        items = self._editor.items()
         return [items[row] for row in rows]
 
     def _get_cell_drag_value(self, row, column):
@@ -332,7 +335,7 @@ class TableModel(QtCore.QAbstractTableModel):
             copied and pasted.
         """
         editor = self._editor
-        item = list(editor.items())[row]
+        item = editor.items()[row]
         drag_value = editor.columns[column].get_drag_value(item)
         return drag_value
 
@@ -360,7 +363,7 @@ class TableModel(QtCore.QAbstractTableModel):
             new_row += 1
 
         # Remove selected rows...
-        items = list(self._editor.items())
+        items = self._editor.items()
         objects = []
         for row in current_rows:
             if row <= new_row:
@@ -414,7 +417,7 @@ class SortFilterTableModel(QtGui.QSortFilterProxyModel):
 
         editor = self._editor
         column = editor.columns[left_mi.column()]
-        items = list(editor.items())
+        items = editor.items()
         left, right = items[left_mi.row()], items[right_mi.row()]
 
         return column.key(left) < column.key(right)
