@@ -23,7 +23,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
-
+from __future__ import absolute_import
 
 import os
 
@@ -37,6 +37,7 @@ from traits.api import (Any, Bool, Callable, Event, HasStrictTraits, Instance,
 from traitsui.tabular_adapter import TabularAdapter
 from .editor import Editor
 from .tabular_model import TabularModel
+import six
 
 
 class HeaderEventFilter(QtCore.QObject):
@@ -100,6 +101,9 @@ class TabularEditor(Editor):
     # The event triggering scrolling.
     scroll_to_row = Event(Int)
 
+    # The event triggering scrolling.
+    scroll_to_column = Event(Int)
+
     # Is the tabular editor scrollable? This value overrides the default.
     scrollable = True
 
@@ -180,6 +184,8 @@ class TabularEditor(Editor):
             'to')
         self.sync_value(factory.scroll_to_row, 'scroll_to_row', 'from',
                         is_event=True)
+        self.sync_value(factory.scroll_to_column, 'scroll_to_column', 'from',
+                        is_event=True)
 
         # Connect other signals as necessary
         control.activated.connect(self._on_activate)
@@ -278,7 +284,7 @@ class TabularEditor(Editor):
         old = self._no_notify
         self._no_notify = True
         try:
-            for name, value in list(keywords.items()):
+            for name, value in keywords.items():
                 setattr(self, name, value)
         finally:
             self._no_notify = old
@@ -321,7 +327,7 @@ class TabularEditor(Editor):
     def _get_image(self, image):
         """ Converts a user specified image to a QIcon.
         """
-        if isinstance(image, str):
+        if isinstance(image, six.string_types):
             self.image = image
             image = self.image
 
@@ -432,6 +438,16 @@ class TabularEditor(Editor):
             self.factory.scroll_to_row_hint, self.control.PositionAtCenter)
         self.control.scrollTo(self.model.index(row, 0), scroll_hint)
 
+    def _scroll_to_column_changed(self, column):
+        """ Scroll to the given column.
+        """
+        scroll_hint = self.scroll_to_row_hint_map.get(
+            self.factory.scroll_to_row_hint, self.control.PositionAtCenter)
+        self.control.scrollTo(
+            self.model.index(self.activated_row, column),
+            scroll_hint
+        )
+
     #-- Table Control Event Handlers -----------------------------------------
 
     def _on_activate(self, index):
@@ -633,7 +649,12 @@ class _TableView(QtGui.QTableView):
             # we make a reasonable guess based on the minimum size hint and the font
             # of the first row.
             size = vheader.minimumSectionSize()
-            font = editor.adapter.get_font(editor.object, editor.name, 0)
+
+            # Check if any columns have been added, and use their font, otherwise
+            # use the default font
+            font = None
+            if 0 in editor.adapter.column_map:
+                font = editor.adapter.get_font(editor.object, editor.name, 0)
             if font is not None:
                 size = max(
                     size, QtGui.QFontMetrics(
@@ -666,6 +687,8 @@ class _TableView(QtGui.QTableView):
             self.viewport().setAcceptDrops(True)
         if factory.drag_move:
             self.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
+        else:
+            self.setDragDropMode(QtGui.QAbstractItemView.DragDrop)
         self.setDropIndicatorShown(True)
 
     def keyPressEvent(self, event):
