@@ -22,7 +22,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
-
+from __future__ import absolute_import
 
 from traits.api import (
     Any,
@@ -44,6 +44,7 @@ from traits.api import (
     cached_property,
     on_trait_change,
     provides)
+import six
 
 #-------------------------------------------------------------------------
 #  'ITabularAdapter' interface:
@@ -607,7 +608,7 @@ class TabularAdapter(HasPrivateTraits):
     def _get_column_dict(self):
         cols = {}
         for i, value in enumerate(self.columns):
-            if isinstance(value, str):
+            if isinstance(value, six.string_types):
                 cols.update({value: value})
             else:
                 cols.update({value[0]: value[1]})
@@ -617,7 +618,7 @@ class TabularAdapter(HasPrivateTraits):
     def _get_column_map(self):
         map = []
         for i, value in enumerate(self.columns):
-            if isinstance(value, str):
+            if isinstance(value, six.string_types):
                 map.append(i)
             else:
                 map.append(value[1])
@@ -641,7 +642,7 @@ class TabularAdapter(HasPrivateTraits):
     def _get_label_map(self):
         map = []
         for i, value in enumerate(self.columns):
-            if isinstance(value, str):
+            if isinstance(value, six.string_types):
                 map.append(value)
             else:
                 map.append(value[0])
@@ -655,7 +656,7 @@ class TabularAdapter(HasPrivateTraits):
         for adapter in self.adapters:
             indices = []
             for label in adapter.columns:
-                if not isinstance(label, str):
+                if not isinstance(label, six.string_types):
                     label = label[0]
 
                 indices.append(labels.index(label))
@@ -670,7 +671,7 @@ class TabularAdapter(HasPrivateTraits):
             mapping = {}
             for label in adapter.columns:
                 id = None
-                if not isinstance(label, str):
+                if not isinstance(label, six.string_types):
                     label, id = label
 
                 key = labels.index(label)
@@ -715,11 +716,11 @@ class TabularAdapter(HasPrivateTraits):
                     get_name = '%s_%s' % (column_id, trait_name)
                     if adapter.trait(get_name) is not None:
                         if prefix == 'get_':
-                            handler = lambda: getattr(adapter.set(
+                            handler = lambda: getattr(adapter.trait_set(
                                 row=self.row, column=column_id,
                                 item=self.item), get_name)
                         else:
-                            handler = lambda: setattr(adapter.set(
+                            handler = lambda: setattr(adapter.trait_set(
                                 row=self.row, column=column_id,
                                 item=self.item), get_name, self.value)
 
