@@ -23,7 +23,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
-
+from __future__ import absolute_import
 
 import shelve
 import os
@@ -111,7 +111,7 @@ class UI(HasPrivateTraits):
     history = Any
 
     # The KeyBindings object (if any) for this UI:
-    key_bindings = Property  # Instance( KeyBindings )
+    key_bindings = Property(depends_on=['view._key_bindings', 'context'])
 
     # The unique ID for this UI for persistence
     id = Str
@@ -194,12 +194,15 @@ class UI(HasPrivateTraits):
     # its value is arrived at too late to be of use in building the UI.
     _scrollable = Bool(False)
 
+    # Cache for key bindings.
+    _key_bindings = Instance('traitsui.key_bindings.KeyBindings')
+
     # List of traits that are reset when a user interface is recycled
     # (i.e. rebuilt).
     recyclable_traits = [
         '_context', '_revert', '_defined', '_visible', '_enabled', '_checked',
         '_search', '_dispatchers', '_editors', '_names', '_active_group',
-        '_undoable', '_rebuild', '_groups_cache'
+        '_undoable', '_rebuild', '_groups_cache', '_key_bindings'
     ]
 
     # List of additional traits that are discarded when a user interface is
@@ -281,7 +284,7 @@ class UI(HasPrivateTraits):
 
         # Make sure that 'visible', 'enabled', and 'checked' handlers are not
         # called after the editor has been disposed:
-        for object in list(self.context.values()):
+        for object in self.context.values():
             object.on_trait_change(self._evaluate_when, remove=True)
 
         # Notify the handler that the view has been closed:
@@ -296,8 +299,8 @@ class UI(HasPrivateTraits):
         self.control = None
 
         # Dispose of any KeyBindings object we reference:
-        if self.key_bindings is not None:
-            self.key_bindings.dispose()
+        if self._key_bindings is not None:
+            self._key_bindings.dispose()
 
         # Break the linkage to any objects in the context dictionary:
         self.context.clear()
@@ -411,7 +414,7 @@ class UI(HasPrivateTraits):
             created.
         """
         # Invoke all of the editor 'name_defined' methods we've accumulated:
-        info = self.info.set(initialized=False)
+        info = self.info.trait_set(initialized=False)
         for method in self._defined:
             method(info)
 
@@ -465,7 +468,7 @@ class UI(HasPrivateTraits):
         if (len(self._visible) +
             len(self._enabled) +
                 len(self._checked)) > 0:
-            for object in list(context.values()):
+            for object in context.values():
                 object.on_trait_change(self._evaluate_when, dispatch='ui')
             self._do_evaluate_when(at_init=True)
 
@@ -479,14 +482,14 @@ class UI(HasPrivateTraits):
     def sync_view(self):
         """ Synchronize context object traits with view editor traits.
         """
-        for name, object in list(self.context.items()):
+        for name, object in self.context.items():
             self._sync_view(name, object, 'sync_to_view', 'from')
             self._sync_view(name, object, 'sync_from_view', 'to')
             self._sync_view(name, object, 'sync_with_view', 'both')
 
     def _sync_view(self, name, object, metadata, direction):
         info = self.info
-        for trait_name, trait in list(object.traits(**{metadata: is_str}).items()):
+        for trait_name, trait in object.traits(**{metadata: is_str}).items():
             for sync in getattr(trait, metadata).split(','):
                 try:
                     editor_id, editor_name = [item.strip()
@@ -809,7 +812,7 @@ class UI(HasPrivateTraits):
         name = 'object'
         n = len(context)
         if (n == 2) and ('handler' in context):
-            for name, value in list(context.items()):
+            for name, value in context.items():
                 if name != 'handler':
                     break
         elif n == 1:
@@ -950,21 +953,24 @@ class UI(HasPrivateTraits):
 
     #-- Property Implementations ---------------------------------------------
 
-    @property_depends_on('view, context')
     def _get_key_bindings(self):
-        view, context = self.view, self.context
-        if (view is None) or (context is None):
-            return None
+        if self._key_bindings is None:
+            # create a new key_bindings instance lazily
 
-        # Get the KeyBindings object to use:
-        values = list(context.values())
-        key_bindings = view.key_bindings
-        if key_bindings is None:
-            from .key_bindings import KeyBindings
+            view, context = self.view, self.context
+            if (view is None) or (context is None):
+                return None
 
-            return KeyBindings(controllers=values)
+            # Get the KeyBindings object to use:
+            values = list(context.values())
+            key_bindings = view.key_bindings
+            if key_bindings is None:
+                from .key_bindings import KeyBindings
+                self._key_bindings = KeyBindings(controllers=values)
+            else:
+                self._key_bindings = key_bindings.clone(controllers=values)
 
-        return key_bindings.clone(controllers=values)
+        return self._key_bindings
 
     #-- Traits Event Handlers ------------------------------------------------
 
