@@ -23,7 +23,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
-
+from __future__ import absolute_import
 
 from traits.api import Instance
 
@@ -56,7 +56,7 @@ class UIEditor(Editor):
         """ Finishes initializing the editor by creating the underlying toolkit
             widget.
         """
-        self.editor_ui = self.init_ui(parent).set(parent=self.ui)
+        self.editor_ui = self.init_ui(parent).trait_set(parent=self.ui)
         self.control = self.editor_ui.control
 
     #-------------------------------------------------------------------------
