@@ -23,6 +23,7 @@ wxPython user interface toolkit.
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
 import logging
 
 import wx
@@ -45,6 +46,7 @@ from .editor \
 from .helper \
     import TraitsUIPanel
 from functools import reduce
+import six
 
 logger = logging.getLogger(__name__)
 
@@ -103,7 +105,7 @@ class SimpleEditor(EditorWithList):
         """ Handles updates to the list of legal checklist values.
         """
         sv = self.string_value
-        if (len(values) > 0) and isinstance(values[0], str):
+        if (len(values) > 0) and isinstance(values[0], six.string_types):
             values = [(x, sv(x, capitalize)) for x in values]
         self.values = valid_values = [x[0] for x in values]
         self.names = [x[1] for x in values]
@@ -120,7 +122,7 @@ class SimpleEditor(EditorWithList):
                     logger.warn('Unable to remove non-current value [%s] from '
                                 'values %s', cur_value[i], values)
         if modified:
-            if isinstance(self.value, str):
+            if isinstance(self.value, six.string_types):
                 cur_value = ','.join(cur_value)
             self.value = cur_value
 
@@ -266,7 +268,7 @@ class CustomEditor(SimpleEditor):
             cur_value.append(control.value)
         else:
             cur_value.remove(control.value)
-        if isinstance(self.value, str):
+        if isinstance(self.value, six.string_types):
             cur_value = ','.join(cur_value)
         self.value = cur_value
 
