@@ -23,6 +23,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
 from operator import itemgetter
 
 import wx
@@ -48,6 +49,8 @@ from .constants \
 
 from .editor \
     import Editor
+import six
+
 
 #-------------------------------------------------------------------------
 #  Trait definitions:
@@ -189,8 +192,8 @@ def find_closest_display(x, y):
         else:
             def _distance(x, y, display):
                 dis_x, dis_y, dis_w, dis_h = display.GetGeometry()
-                dis_mid_x = (dis_x + dis_w) / 2
-                dis_mid_y = (dis_y + dis_h) / 2
+                dis_mid_x = dis_x + dis_w / 2
+                dis_mid_y = dis_y + dis_h / 2
 
                 return (x - dis_mid_x)**2 + (y - dis_mid_y)**2
 
@@ -284,7 +287,7 @@ def enum_values_changed(values):
     """
 
     if isinstance(values, dict):
-        data = [(str(v), n) for n, v in list(values.items())]
+        data = [(six.text_type(v), n) for n, v in values.items()]
         if len(data) > 0:
             data.sort(key=itemgetter(0))
             col = data[0][0].find(':') + 1
@@ -297,12 +300,12 @@ def enum_values_changed(values):
         if not isinstance(handler, BaseTraitHandler):
             raise TraitError("Invalid value for 'values' specified")
         if handler.is_mapped:
-            data = [(str(n), n) for n in list(handler.map.keys())]
+            data = [(six.text_type(n), n) for n in handler.map.keys()]
             data.sort(key=itemgetter(0))
         else:
-            data = [(str(v), v) for v in handler.values]
+            data = [(six.text_type(v), v) for v in handler.values]
     else:
-        data = [(str(v), v) for v in values]
+        data = [(six.text_type(v), v) for v in values]
 
     names = [x[0] for x in data]
     mapping = {}
@@ -564,7 +567,7 @@ class GroupEditor(Editor):
     def __init__(self, **traits):
         """ Initializes the object.
         """
-        self.set(**traits)
+        self.trait_set(**traits)
 
 #-------------------------------------------------------------------------
 #  'PopupControl' class:
