@@ -22,6 +22,8 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
+
 from operator import itemgetter
 
 import wx
@@ -68,7 +70,7 @@ from .table_model \
     import TableModel, TraitGridSelection
 
 from .helper import TraitsUIPanel
-from functools import reduce
+
 
 #-------------------------------------------------------------------------
 #  Constants:
@@ -285,12 +287,6 @@ class TableEditor(Editor, BaseTableEditor):
             getattr(self, '_selection_%s_updated' % mode),
             'selection_changed', dispatch='ui')
 
-        # Make sure the selection is initialized:
-        if row_mode:
-            self.set_selection(items[0:1])
-        else:
-            self.set_selection()
-
         # Set the min height of the grid panel to 0, this will provide
         # a scrollbar if the window is resized such that only the first row
         # is visible
@@ -405,7 +401,7 @@ class TableEditor(Editor, BaseTableEditor):
                 self.toolbar_ui = ui = view.ui(
                     context={'object': self, 'model': self.model},
                     parent=parent,
-                    kind='subpanel').set(
+                    kind='subpanel').trait_set(
                     parent=self.ui)
                 tb_sizer.Add(ui.control, 0)
 
@@ -593,8 +589,9 @@ class TableEditor(Editor, BaseTableEditor):
                 return [index]
 
         elif sm == 'cells':
-            return list(set([row_col[0] for
-                             row_col in self.selected_cell_indices]))
+            return list(
+                {row_col[0] for row_col in self.selected_cell_indices}
+            )
 
         elif sm == 'cell':
             index = self.selected_cell_index[0]
@@ -614,8 +611,9 @@ class TableEditor(Editor, BaseTableEditor):
                 return [item]
 
         elif sm == 'cells':
-            return list(set([item_name[0]
-                             for item_name in self.selected_cells]))
+            return list(
+                {item_name[0] for item_name in self.selected_cells}
+            )
 
         elif sm == 'cell':
             item = self.selected_cell[0]
@@ -697,7 +695,7 @@ class TableEditor(Editor, BaseTableEditor):
             structure = prefs.get('structure')
             if (structure is not None) and (factory.edit_view != ' '):
                 self.control.GetSizer().SetStructure(self.control, structure)
-        except:
+        except Exception:
             pass
 
     #-------------------------------------------------------------------------
@@ -769,8 +767,7 @@ class TableEditor(Editor, BaseTableEditor):
             index, row = -1, None
 
         # Save the new selection information:
-        self.set(selected_row_index=index,
-                 trait_change_notify=False)
+        self.trait_set(selected_row_index=index, trait_change_notify=False)
         self.setx(selected_row=row)
 
         # Update the toolbar status:
@@ -835,8 +832,7 @@ class TableEditor(Editor, BaseTableEditor):
             index, column = -1, ''
 
         # Save the new selection information:
-        self.set(selected_column_index=index,
-                 trait_change_notify=False)
+        self.trait_set(selected_column_index=index, trait_change_notify=False)
         self.setx(selected_column=column)
 
         # Invoke the user 'on_select' handler:
@@ -860,8 +856,8 @@ class TableEditor(Editor, BaseTableEditor):
         values.sort(key=itemgetter(0))
 
         # Save the new selection information:
-        self.set(selected_column_indices=[v[0] for v in values],
-                 trait_change_notify=False)
+        self.trait_set(selected_column_indices=[v[0] for v in values],
+                       trait_change_notify=False)
         columns = [v[1] for v in values]
         self.setx(selected_columns=columns)
 
@@ -898,8 +894,7 @@ class TableEditor(Editor, BaseTableEditor):
             index, cell = (-1, -1), (None, '')
 
         # Save the new selection information:
-        self.set(selected_cell_index=index,
-                 trait_change_notify=False)
+        self.trait_set(selected_cell_index=index, trait_change_notify=False)
         self.setx(selected_cell=cell)
 
         # Update the toolbar status:
@@ -974,7 +969,8 @@ class TableEditor(Editor, BaseTableEditor):
                 self.set_extended_selection()
             else:
                 self.set_extended_selection(
-                    (None, self.model.get_column_name[index]))
+                    (None, self.model.get_column_name(index))
+                )
 
     def _selected_columns_changed(self, names):
         if not self._no_notify:
@@ -997,7 +993,8 @@ class TableEditor(Editor, BaseTableEditor):
             else:
                 self.set_extended_selection(
                     (self.value[row],
-                     self.model.get_column_name[column]))
+                     self.model.get_column_name(column))
+                )
 
     def _selected_cells_changed(self, cells):
         if not self._no_notify:
@@ -1029,17 +1026,17 @@ class TableEditor(Editor, BaseTableEditor):
 
                 deletable = self.factory.deletable
                 if delete.enabled and callable(deletable):
-                    delete.enabled = reduce(
-                        lambda l, r: l and r, [
-                            deletable(item) for item in self.selected_items], True)
+                    delete.enabled = all(
+                        deletable(item) for item in self.selected_items
+                    )
 
-                toolbar.search.enabled = toolbar.add.enabled = True
+                toolbar.add.enabled = toolbar.search.enabled = no_filter
                 toolbar.move_up.enabled = (no_filter and (start > 0))
                 toolbar.move_down.enabled = (no_filter and (indices[-1] < n))
             else:
-                toolbar.add.enabled = no_filter
-                toolbar.search.enabled  = toolbar.delete.enabled    = \
-                    toolbar.move_up.enabled = toolbar.move_down.enabled = False
+                toolbar.add.enabled = toolbar.search.enabled = no_filter
+                toolbar.delete.enabled = toolbar.move_up.enabled = \
+                    toolbar.move_down.enabled = False
 
     #-------------------------------------------------------------------------
     #  Handles the contents of the model being resorted:
@@ -1190,8 +1187,8 @@ class TableEditor(Editor, BaseTableEditor):
         self.add_row()
 
     #-------------------------------------------------------------------------
-    #  Handles the user requesting to delete the currently selected items of the
-    #  table:
+    #  Handles the user requesting to delete the currently selected items of
+    #  the table:
     #-------------------------------------------------------------------------
 
     def on_delete(self):
@@ -1229,11 +1226,11 @@ class TableEditor(Editor, BaseTableEditor):
         if n > 0:
             if self.in_row_mode:
                 self.set_selection(
-                    list(set([items[i] for i in indices])))
+                    list({items[i] for i in indices}))
             else:
-                self.set_extended_selection(
-                    list(set([(items[indices[i]], values[i][1])
-                              for i in range(n)])))
+                self.set_extended_selection(list(
+                    {(items[indices[i]], values[i][1]) for i in range(n)}
+                ))
         else:
             self._update_toolbar(False)
 
@@ -1247,9 +1244,11 @@ class TableEditor(Editor, BaseTableEditor):
             table.
         """
         columns = self.columns[:]
-        columns.extend([c
-                        for c in (self.factory.columns + self.factory.other_columns)
-                        if c not in columns])
+        columns.extend([
+            c
+            for c in (self.factory.columns + self.factory.other_columns)
+            if c not in columns
+        ])
         self.edit_traits(
             parent=self.control,
             view=View([Item('columns',
@@ -1288,7 +1287,7 @@ class TableEditor(Editor, BaseTableEditor):
         """
         self._no_notify = True
 
-        for name, value in list(keywords.items()):
+        for name, value in keywords.items():
             setattr(self, name, value)
 
         self._no_notify = False
@@ -1685,7 +1684,7 @@ class TableSearchHandler(Handler):
             editor = self.editor
             filter = info.object.filter
             items = [item for item in editor.model.get_filtered_items()
-                     if list(filter(item))]
+                     if filter(item)]
             editor.set_selection(items)
 
             if len(items) == 1:
@@ -1701,10 +1700,12 @@ class TableSearchHandler(Handler):
         """ Handles the user clicking the OK button.
         """
         if info.initialized:
-            info.ui.dispose()
+            self.close(info, True)
+
 
 # Define the SimpleEditor class.
 SimpleEditor = TableEditor
 
+
 # Define the ReadonlyEditor class.
 ReadonlyEditor = TableEditor
