@@ -22,9 +22,11 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
 import os
 import wx
 import copy
+import six
 
 try:
     from pyface.wx.drag_and_drop import PythonDropSource, \
@@ -651,7 +653,7 @@ class SimpleEditor(Editor):
             return -1
 
         icon_name = node.get_icon(object, is_expanded)
-        if isinstance(icon_name, str):
+        if isinstance(icon_name, six.string_types):
             if icon_name.startswith('@'):
                 image = convert_image(icon_name, 3)
                 if image is None:
@@ -662,7 +664,7 @@ class SimpleEditor(Editor):
                     path = self
                 else:
                     path = node.get_icon_path(object)
-                    if isinstance(path, str):
+                    if isinstance(path, six.string_types):
                         path = [path, node]
                     else:
                         path.append(node)
