@@ -22,6 +22,7 @@
 #  Imports:
 #-------------------------------------------------------------------------
 
+from __future__ import absolute_import
 import wx
 
 from .helper \
@@ -256,7 +257,7 @@ class ModalDialog(BaseDialog):
         """ Creates a copy of a *context* dictionary.
         """
         result = {}
-        for name, value in list(context.items()):
+        for name, value in context.items():
             if value is not None:
                 result[name] = value.clone_traits()
             else:
@@ -271,7 +272,7 @@ class ModalDialog(BaseDialog):
     def _apply_context(self, from_context, to_context):
         """ Applies the traits in the *from_context* to the *to_context*.
         """
-        for name, value in list(from_context.items()):
+        for name, value in from_context.items():
             if value is not None:
                 to_context[name].copy_traits(value)
             else:
